#ifndef DISKDUMP_FORMAT_H
#define DISKDUMP_FORMAT_H

/* $Id: diskdump_format.h,v 1.37 2006/04/17 15:47:29 akira Exp $ */

#include "diskdumpcmn.h"

enum {
	DISKDUMP_FORMAT_VER_0,
	DISKDUMP_FORMAT_VER_1
};

#define CURRENT_DISKDUMP_FORMAT_VER	DISKDUMP_FORMAT_VER_1

#define MINIMUM_FORMAT_SIZE	5

#define SAMPLE_RATE	"sample_rate"
#define BLOCK_ORDER	"block_order"
#define TOTAL_BLOCKS	"total_blocks"
#define DUMP_LEVEL	"dump_level"
#define COMPRESS	"compress"

#define Formula(x)	(((x) + 7) * 11)

enum {
	FORMATTED,
	HAS_PANIC_DUMP,
	NOT_A_DUMP_DEVICE,
	FORMAT_REQUIRED,
	NOT_SUITABLE_DEVICE,
	SWAP_FORMATTED,
	SWAP_WITH_DUMP,
	MOUNTED,
	HAS_IN_PROGRESS_PANIC_DUMP,
	SWAP_WITH_IN_PROGRESS_DUMP,
};

typedef struct format_params {
	char	signature[SIG_LEN];	/* "diskdump" */
	int32_t	block_number;		/* usually 2 */
	int32_t	fmt_version;		/* 0, 1, ... , N */
	int32_t	sample_rate;		/* the same as the above */
	int32_t	block_order;		/* the same as the above */
	int64_t	format_size;		/* the same as the above */
	int32_t	dump_level;		/* 0 to 31    (only for partial dump) */
	int32_t	compress;		/* 0 or not 0 (only for compression) */
	char	cmd_version[32];	/* expressed by "i.j.k" */
} format_params_t;

typedef struct dump_device {
	const char	*name;		/* the specified device name */
	int		 fd;		/* file descriptor for the device */
	int		 page_size;	/* by sysconf() */
	off_t		 block_size;	/* total blocks of the device */
	format_params_t	*dev;		/* written in the device */
	format_params_t	*req;		/* written in "/proc/diskdump" */
	void		*scratch;	/* buffer */
	bool		 do_progress;	/* show progress bar if true */
	bool		 is_dump_device;/* data in the block #0 */
	bool		 is_swap_device;/* swap partition */
	bool		 has_panic_dump;/* data in the block #1 */
	bool             in_progress;	/* whether dump is in progress */
	bool		 broken_header;	/* data in the block #2 */
	bool		 ver0_formatted;/* data in the block #0 to #2 */
	char		*err_msg;	/* error massage */
	char		*reason;	/* auxiliary to the device status
					   and debugging */
} dump_device_t;

int		 get_module_params(dump_device_t *);
int		 check_device(dump_device_t *, bool ignore);
void		 show_device_info(dump_device_t *, int status, bool verbose);
void		 show_format_info(dump_device_t *, bool verbose);
dump_device_t	*alloc_device(void);
int		 open_device(dump_device_t *, const char *name);
off_t		 format_device(dump_device_t *);
bool		 enough_block_size(dump_device_t *);

#endif
