/*
 *  diskdumpctl_proc.c
 *
 *  Copyright (C) 2004, 2005  FUJITSU LIMITED
 *  Written by Tatsuo Uchida (tuchida@us.fujitsu.com)
 *
 *  Some codes were derived from netdump and copyright belongs to
 *  Red Hat, Inc.
 */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
/* $Id: diskdumpctl_proc.c,v 1.10 2005/05/23 23:31:40 akira Exp $ */

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdbool.h>
#include <errno.h>
#include <fcntl.h>
#include <string.h>
#include <sys/ioctl.h>
#include "version.h"
#include "diskdumpcmn.h"

#define CMD	"diskdumpctl"

#define BLKADDDUMPDEVICE	0xdf00
#define BLKREMOVEDUMPDEVICE	0xdf01

int main(int argc, char *argv[])
{
	int dev_fd, proc_fd;
	int opt;
	bool do_unregister = false, verbose = false;
	const char *dev_name;

	while ((opt = getopt(argc, argv, "uvV")) != -1) {
		switch (opt) {
		case 'u':
			do_unregister = true;
			break;
		case 'v':
			verbose = true;
			break;
		case 'V':
			version(CMD);
			break;
		default:
			usage(CMD);
			break;
		}
	}

	if (optind + 1 != argc)
		usage(CMD);

	dev_name = argv[optind];

	dev_fd = open(dev_name, O_RDONLY);
	if (dev_fd < 0) {
		Syserr("open %s", dev_name);
		exit(1);
	}

	proc_fd = open(_PROC_DISKDUMP, O_RDONLY);
	if (proc_fd < 0) {
		Syserr("open %s", _PROC_DISKDUMP);
		exit(1);
	}

	if (!do_unregister) {
		if (ioctl(proc_fd, BLKADDDUMPDEVICE, dev_fd) < 0) {
			Syserr("add %s", dev_name);
			exit(1);
		}
		Info("%s has been added.\n", dev_name);
	} else {
		if (ioctl(proc_fd, BLKREMOVEDUMPDEVICE, dev_fd) < 0) {
			Syserr("remove %s", dev_name);
			exit(1);
		}
		Info("%s has been removed.\n", dev_name);
	}

	exit(0);
}
