/*
 *  dumpfile.c
 *
 *  Copyright (C) 2005, 2006  FUJITSU LIMITED
 *
 */
/* $Id: dumpfile.c,v 1.4 2006/04/21 19:38:38 akira Exp $ */

#define _GNU_SOURCE

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include "savecore.h"

DumpFile *open_dump_file(char *dir)
{
	DumpFile *dump_file;
	char *filename;
	int fd;

	if ((dump_file = calloc(1, sizeof(DumpFile))) == NULL) {
		return NULL;
	}

	dump_file->dir = dir;

	if (asprintf(&filename, "%s%s", dir, INCOMPLETE_DUMP_FILE) < 0) {
		return NULL;
	}

	fd = open(filename, O_WRONLY|O_TRUNC|O_CREAT, S_IRUSR|S_IWUSR);
	if (fd < 0) {
		free(dump_file);
		free(filename);
		return NULL;
	}

	dump_file->fd	= fd;
	dump_file->name	= filename;

	return dump_file;
}

int rename_dump_file(DumpFile *dump_file)
{
	char *old_file;
	char *new_file;
	int ret;

	if (asprintf(&old_file, "%s%s", dump_file->dir, INCOMPLETE_DUMP_FILE)
			< 0) {
		return FAILED;
	}
	if (asprintf(&new_file, "%s%s", dump_file->dir, COMPLETE_DUMP_FILE)
			< 0) {
		return FAILED;
	}

	ret = rename(old_file, new_file);
	if ((dump_file->name = strdup(new_file)) == NULL) {
		return FAILED;
	}

	free(old_file);
	free(new_file);

	return ret;
}

int write_page_desc(DumpFile *dump_file, page_desc_t *desc)
{
	if (dump_seek(dump_file, dump_file->page_desc_offset) < 0)
		return SEEK_ERROR_IN_DUMP_DEVICE;

	if (write(dump_file->fd, desc, sizeof(*desc)) != sizeof(*desc))
		return WRITE_ERROR_IN_DUMP_FILE;

	dump_file->page_desc_offset += sizeof(*desc);

	return COMPLETE;
}

int write_page_data(DumpFile *dump_file, const void *data, uint32_t size)
{
	if (dump_seek(dump_file, dump_file->data_offset) < 0)
		return SEEK_ERROR_IN_DUMP_DEVICE;

	if (write(dump_file->fd, data, size) != size)
		return WRITE_ERROR_IN_DUMP_FILE;

	dump_file->data_offset += size;

	return COMPLETE;
}

int write_blocks(DumpFile *dump_file, const void *data, uint32_t size)
{
	int block_size = dump_file->device->header->block_size;

	if (dump_seek(dump_file, dump_file->data_offset) < 0)
		return SEEK_ERROR_IN_DUMP_DEVICE;

	if (write(dump_file->fd, data, size) != size)
		return WRITE_ERROR_IN_DUMP_FILE;

	dump_file->data_offset += roundup(size, block_size);

	return COMPLETE;
}
